#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

def binary_string(number: int) -> str: ...
def string_score(string: str) -> int: ...
def is_palindrome(string: str) -> bool: ...
