#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from typing import Any
from typing import Generator
from typing import Iterable
from typing import List
from typing import Union

def chunks(list: List[Any], chunk_size: int) -> Generator: ...
def is_permutation(a: Union[Iterable[Any], int], b: Union[Iterable[Any], int]): ...
def rotate(rlist: Any, rn: int = ..., left_rotate: bool = ...): ...
def rotations_gen(rlist: Union[List[Any], tuple]) -> None: ...
def digits_list(number: int): ...
def int_from_digits(dlist: Union[List[int], tuple, Iterable[int]]): ...
def iter_product(l: Union[List[int], tuple, Iterable[int]]): ...
def files_gen(dirpath:str, abs:bool): ...
def dirs_gen(dirpath:str, abs:bool): ...

